package main

import (
	"encoding/base64"
	"errors"
	"fmt"
	"github.com/xanzy/go-gitlab"
	"log"
	"os"
	"strings"
)

const (
	gitlabToken       string = "glpat-_M8DpM1R5ak5t9kJz35o"
	gitlabUrl         string = "https://penahub.gitlab.yandexcloud.net/api/v4"
	fileName          string = "CHANGELOG.md"
	branchNameMain    string = "main"
	branchNameStaging string = "staging"
)

var (
	notFoundErr = errors.New("404 Not Found")
)

func main() {
	gitClient, err := gitlab.NewClient(gitlabToken, gitlab.WithBaseURL(gitlabUrl))
	if err != nil {
		log.Fatalf("failed create client: %v", err)
	}

	var allProjects []*gitlab.Project
	opts := &gitlab.ListProjectsOptions{
		ListOptions: gitlab.ListOptions{
			PerPage: 100,
		},
	}

	for {
		projects, resp, err := gitClient.Projects.ListProjects(opts)
		if err != nil {
			log.Fatalf("failed get projects: %v", err)
		}
		allProjects = append(allProjects, projects...)
		if resp.NextPage == 0 {
			break
		}
		opts.Page = resp.NextPage
	}

	resultFile, err := os.Create("report.txt")
	if err != nil {
		log.Fatalf("failed create report file: %v", err)
	}

	defer resultFile.Close()

	for _, pj := range allProjects {
		changes, err := getChangelog(gitClient, pj.ID)
		if err != nil {
			log.Printf("Error getting changelogs for %s: %v", pj.Name, err)
			continue
		}

		if len(changes) > 0 {
			fmt.Fprintf(resultFile, "Repository: %s\n", pj.Name)
			for _, change := range changes {
				fmt.Fprintln(resultFile, change)
			}
			fmt.Fprintln(resultFile)
		}
	}
}

func getChangelog(gitClient *gitlab.Client, pjID int) ([]string, error) {
	stagingContent, err := fileContent(gitClient, pjID, fileName, branchNameStaging)
	if err != nil {
		return nil, err
	}

	mainContent, err := fileContent(gitClient, pjID, fileName, branchNameMain)
	if err != nil && !compareError(err, notFoundErr) {
		return nil, err
	}

	mainLines := strings.Split(mainContent, "\n")
	stagingLines := strings.Split(stagingContent, "\n")

	var diffs []string
	for _, line := range stagingLines {
		if !contains(mainLines, line) {
			diffs = append(diffs, line)
		}
	}

	return diffs, nil
}

func fileContent(gitClient *gitlab.Client, pjID int, filePath, branch string) (string, error) {
	file, _, err := gitClient.RepositoryFiles.GetFile(pjID, filePath, &gitlab.GetFileOptions{Ref: gitlab.Ptr(branch)})
	if err != nil {
		return "", err
	}

	data, err := base64.StdEncoding.DecodeString(file.Content)
	if err != nil {
		return "", err
	}

	return string(data), nil
}

func contains(lines []string, line string) bool {
	for _, l := range lines {
		if l == line {
			return true
		}
	}
	return false
}

func compareError(err1 error, err2 error) bool {
	if err1.Error() == err2.Error() {
		return true
	}
	return false
}
