package transfer

import (
	"context"
	"encoding/json"
	"errors"
	"fmt"
	"gitea.pena/PenaSide/hlog"
	"github.com/go-resty/resty/v2"
	"go.mongodb.org/mongo-driver/bson"
	"go.mongodb.org/mongo-driver/mongo"
	"go.mongodb.org/mongo-driver/mongo/options"
	"net/url"
	"time"
	"transfer_to_clickhouse/models"
)

type Customer struct {
	logger            hlog.Logger
	accountCollection *mongo.Collection
	historyCollection *mongo.Collection
	authClient        *AuthClient
}

func NewCustomer(logger hlog.Logger, db *mongo.Database, authUserUrl string) *Customer {
	return &Customer{
		logger:            logger,
		accountCollection: db.Collection("accounts"),
		historyCollection: db.Collection("history"),
		authClient:        NewAuthClient(authUserUrl),
	}
}

type AuthClient struct {
	url        string
	httpClient *resty.Client
}

func NewAuthClient(url string) *AuthClient {
	return &AuthClient{
		url:        url,
		httpClient: resty.New(),
	}
}

func (r *AuthClient) GetUser(userID string) (*User, error) {
	userURL, err := url.JoinPath(r.url, userID)
	if err != nil {
		return nil, err
	}
	var user User
	resp, err := r.httpClient.R().
		SetResult(&user).
		Get(userURL)
	if err != nil {
		return nil, err
	}

	if resp.StatusCode() != 200 {
		return nil, fmt.Errorf("failed to fetch user: %s", resp.Status())
	}

	return &user, nil
}

type User struct {
	ID          string     `json:"_id"`
	Login       string     `json:"login"`
	Email       string     `json:"email"`
	PhoneNumber string     `json:"phoneNumber"`
	IsDeleted   bool       `json:"isDeleted"`
	CreatedAt   time.Time  `json:"createdAt"`
	UpdatedAt   time.Time  `json:"updatedAt"`
	DeletedAt   *time.Time `json:"deletedAt,omitempty"`
}

type Account struct {
	ID        string        `json:"id" bson:"_id,omitempty"`
	UserID    string        `json:"userId" bson:"userId"`
	Cart      []string      `json:"cart" bson:"cart"`
	Wallet    Wallet        `json:"wallet" bson:"wallet"`
	Name      Name          `json:"name" bson:"name"`
	Status    AccountStatus `json:"status" bson:"status"`
	Deleted   bool          `json:"isDeleted" bson:"isDeleted"`
	CreatedAt time.Time     `json:"createdAt" bson:"createdAt"`
	UpdatedAt time.Time     `json:"updatedAt" bson:"updatedAt"`
	DeletedAt *time.Time    `json:"deletedAt,omitempty" bson:"deletedAt,omitempty"`
	From      string        `json:"from" bson:"from"`
	Partner   string        `json:"partner" bson:"partner"`
	Version   uint32        `json:"version" bson:"version"`
}

type Wallet struct {
	Cash            int64  `json:"cash" bson:"cash"`
	Currency        string `json:"currency" bson:"currency"`
	Spent           int64  `json:"spent" bson:"spent"`
	PurchasesAmount int64  `json:"purchasesAmount" bson:"purchasesAmount"`
	Money           int64  `json:"money" bson:"money"`
	LastPaymentID   string `json:"lastPaymentId" bson:"lastPaymentId"`
}

type Name struct {
	Middlename string `json:"middlename,omitempty"`
	FirstName  string `json:"firstname,omitempty"`
	Orgname    string `json:"orgname,omitempty"`
	Secondname string `json:"secondname,omitempty"`
}

type AccountStatus string

const (
	AccountStatusNko     AccountStatus = "nko"
	AccountStatusNo      AccountStatus = "no"
	AccountStatusOrg     AccountStatus = "org"
	DefaultAccountStatus AccountStatus = AccountStatusNo
)

type History struct {
	ID         string     `json:"id" bson:"_id,omitempty"`
	UserID     string     `json:"userId" bson:"userId"`
	Comment    string     `json:"comment" bson:"comment"`
	Key        string     `json:"key" bson:"key"`
	RawDetails RawDetails `json:"rawDetails" bson:"rawDetails"`
	Deleted    bool       `json:"isDeleted" bson:"isDeleted"`
	CreatedAt  time.Time  `json:"createdAt" bson:"createdAt"`
	UpdatedAt  time.Time  `json:"updatedAt" bson:"updatedAt"`
	DeletedAt  *time.Time `json:"deletedAt,omitempty" bson:"deletedAt,omitempty"`
	Version    uint32     `json:"version" bson:"version"`
}

type RawDetails struct {
	Tariffs []Tariff `json:"tariffs" bson:"tariffs"`
	Price   int64    `json:"price" bson:"price"`
	Comment string   `json:"comment" bson:"comment"`
}

type Tariff struct {
	ID         string      `json:"_id"`
	Name       string      `json:"name"`
	Price      uint64      `json:"price,omitempty"`
	IsCustom   bool        `json:"isCustom"`
	Privileges []Privilege `json:"privileges"`
	Deleted    bool        `json:"isDeleted"`
	CreatedAt  time.Time   `json:"createdAt"`
	UpdatedAt  time.Time   `json:"updatedAt"`
	DeletedAt  *time.Time  `json:"deletedAt,omitempty"`
}

type Privilege struct {
	ID          string        `json:"_id"`
	Name        string        `json:"name"`
	PrivilegeID string        `json:"privilegeId"`
	ServiceKey  string        `json:"serviceKey"`
	Description string        `json:"description"`
	Amount      uint64        `json:"amount"`
	Type        PrivilegeType `json:"type"`
	Value       string        `json:"value"`
	Price       uint64        `json:"price"`
}

type PrivilegeType string

const (
	PrivilegeTypeCount = "count"
	PrivilegeTypeDay   = "day"
	PrivilegeTypeFull  = "full"
)

func (r *Customer) SetAccountData(ctx context.Context) error {
	filter := bson.M{"isDeleted": false}
	opts := options.Find()
	cursor, err := r.accountCollection.Find(ctx, filter, opts)
	if err != nil {
		return err
	}
	defer cursor.Close(ctx)

	var accounts []Account
	for cursor.Next(ctx) {
		var account Account
		if err := cursor.Decode(&account); err != nil {
			return err
		}
		accounts = append(accounts, account)
	}

	if err := cursor.Err(); err != nil {
		return err
	}

	for _, account := range accounts {
		r.logger.Emit(models.InfoGetAccount{
			CtxUserID:    account.UserID,
			CtxAccountID: account.ID,
		})

		for _, item := range account.Cart {
			r.logger.Emit(models.InfoAddToCart{
				CtxUserID:    account.UserID,
				CtxAccountID: account.ID,
				CtxTariffID:  item,
			})
		}

		userData, err := r.authClient.GetUser(account.UserID)
		if err != nil {
			fmt.Printf("error getting user from auth service,scip him, userid:%s, error: %s", account.UserID, err.Error())
			continue
		}
		quiz := ""
		if account.From != "" {
			quiz = "quiz"
		}

		r.logger.Emit(models.InfoCreateAccount{
			CtxUserID:      account.UserID,
			CtxAccountID:   account.ID,
			KeyFromSource:  quiz,
			KeyFromID:      account.From,
			KeyFromPartner: account.Partner,
			CtxLogin:       userData.Login,
			CtxEmail:       userData.Email,
			CtxPhone:       userData.PhoneNumber,
			KeyCurrency:    account.Wallet.Currency,
		})
	}

	return nil
}

func (r *Customer) SetPayCartData(ctx context.Context) error {
	filter := bson.M{"isDeleted": false}
	opts := options.Find()
	cursor, err := r.historyCollection.Find(ctx, filter, opts)
	if err != nil {
		return err
	}
	defer cursor.Close(ctx)

	var histories []History
	for cursor.Next(ctx) {
		var history History
		if err := cursor.Decode(&history); err != nil {
			return err
		}
		histories = append(histories, history)
	}

	if err := cursor.Err(); err != nil {
		return err
	}

	for _, history := range histories {
		currentAccount, err := r.findByUserID(ctx, history.UserID)
		if err != nil {
			fmt.Printf("Error getting account by userID:%s", err.Error())
			continue
		}

		r.logger.Emit(models.InfoPayCart{
			CtxUserID:    history.UserID,
			CtxAccountID: currentAccount.ID,
			KeySuccess:   uint8(1),
			CtxPrice:     history.RawDetails.Price,
			//CtxTariff:    strings.Join(account.Cart, ","),
			//CtxDiscount:  strings.Join(utils.GetAppliedDiscountsIDs(discountResponse.AppliedDiscounts), ","),
			//CtxRowPrice:  int64(tariffsAmount),
			CtxRowData: marshalRawDetails(history.RawDetails),
		})
	}

	return nil
}

func (r *Customer) findByUserID(ctx context.Context, userID string) (*Account, error) {
	filter := bson.M{
		"userId":    userID,
		"isDeleted": false,
	}
	var account Account
	err := r.accountCollection.FindOne(ctx, filter).Decode(&account)
	if err != nil {
		if errors.Is(err, mongo.ErrNoDocuments) {
			fmt.Printf("No account found for userID: %s", userID)
			return nil, nil
		}
		return nil, err
	}

	return &account, nil
}

func marshalRawDetails(details RawDetails) string {
	data, err := json.Marshal(details)
	if err != nil {
		return ""
	}
	return string(data)
}
