package transfer

import (
	"context"
	"database/sql"
	"encoding/json"
	"fmt"
	"gitea.pena/PenaSide/hlog"
	"github.com/lib/pq"
	"time"
	"transfer_to_clickhouse/models"
)

type QuizAnswerer struct {
	db     *sql.DB
	logger hlog.Logger
}

func NewQuizAnswerer(db *sql.DB, logger hlog.Logger) *QuizAnswerer {
	return &QuizAnswerer{
		db:     db,
		logger: logger,
	}
}

type Answer struct {
	ID          int64           `db:"id" json:"id"`
	Content     *string         `db:"content" json:"content"`
	QuizID      int64           `db:"quiz_id" json:"quiz_id"`
	QuestionID  int64           `db:"question_id" json:"question_id"`
	Fingerprint *string         `db:"fingerprint" json:"fingerprint"`
	Session     *string         `db:"session" json:"session"`
	CreatedAt   time.Time       `db:"created_at" json:"created_at"`
	Result      bool            `db:"result" json:"result"`
	New         bool            `db:"new" json:"new"`
	Deleted     bool            `db:"deleted" json:"deleted"`
	Email       string          `db:"email" json:"email"`
	DeviceType  string          `db:"device_type" json:"device_type"`
	Device      string          `db:"device" json:"device"`
	Os          string          `db:"os" json:"os"`
	Browser     string          `db:"browser" json:"browser"`
	Ip          string          `db:"ip" json:"ip"`
	Start       bool            `db:"start" json:"start"`
	Utm         json.RawMessage `db:"utm" json:"utm"`
	Version     int32           `db:"version" json:"version"`
}

func (r *QuizAnswerer) AnswererEvents(ctx context.Context) error {
	query := `SELECT id, content, quiz_id, question_id, fingerprint, session, created_at, result, new, deleted,
                  email, device_type, device, os, browser, ip, start, utm, version 
              FROM answer`

	rows, err := r.db.QueryContext(ctx, query)
	if err != nil {
		return err
	}
	defer rows.Close()

	var answers []Answer

	for rows.Next() {
		var answer Answer
		if err := rows.Scan(&answer.ID, &answer.Content, &answer.QuizID, &answer.QuestionID, &answer.Fingerprint,
			&answer.Session, &answer.CreatedAt, &answer.Result, &answer.New, &answer.Deleted, &answer.Email,
			&answer.DeviceType, &answer.Device, &answer.Os, &answer.Browser, &answer.Ip, &answer.Start,
			&answer.Utm, &answer.Version); err != nil {
			return err
		}
		answers = append(answers, answer)
	}

	if err := rows.Err(); err != nil {
		return err
	}

	for _, ans := range answers {
		quiz, err := r.getQuizByID(ctx, ans.QuizID)
		if err != nil {
			fmt.Println(err.Error())
			continue
		}

		if ans.Start {
			r.logger.Emit(models.InfoQuizOpen{
				KeyOS:         ans.Os,
				KeyDevice:     ans.Device,
				KeyDeviceType: ans.DeviceType,
				KeyBrowser:    ans.Browser,
				CtxQuiz:       quiz.Qid.String(),
				CtxQuizID:     quiz.ID,
				CtxIDInt:      ans.ID,
				CtxSession:    *ans.Session,
			})
			continue
		}

		if ans.Result {
			r.logger.Emit(models.InfoResult{
				KeyOS:         ans.Os,
				KeyDevice:     ans.Device,
				KeyDeviceType: ans.DeviceType,
				KeyBrowser:    ans.Browser,
				CtxQuiz:       quiz.Qid.String(),
				CtxQuizID:     quiz.ID,
				CtxQuestionID: ans.QuestionID,
				CtxIDInt:      ans.ID,
				CtxSession:    *ans.Session,
			})
			continue
		}
		r.logger.Emit(models.InfoAnswer{
			KeyOS:         ans.Os,
			KeyDevice:     ans.Device,
			KeyDeviceType: ans.DeviceType,
			KeyBrowser:    ans.Browser,
			CtxQuiz:       quiz.Qid.String(),
			CtxQuizID:     quiz.ID,
			CtxQuestionID: ans.QuestionID,
			CtxIDInt:      ans.ID,
			CtxSession:    *ans.Session,
		})
	}

	return nil
}

func (r *QuizAnswerer) getQuizByID(ctx context.Context, quizID int64) (*Quiz, error) {
	query := `
		SELECT id, qid, accountid, deleted, archived, fingerprinting, repeatable, note_prevented, 
			mail_notifications, unique_answers, super, group_id, name, description, config, status, 
			limit_answers, due_to, time_of_passing, pausable, version, version_comment, parent_ids, 
			created_at, updated_at, questions_count, answers_count, average_time_passing, sessions_count
		FROM quiz where id = $1
	`

	var quiz Quiz

	row := r.db.QueryRowContext(ctx, query, quizID)
	err := row.Scan(
		&quiz.ID,
		&quiz.Qid,
		&quiz.Accountid,
		&quiz.Deleted,
		&quiz.Archived,
		&quiz.Fingerprinting,
		&quiz.Repeatable,
		&quiz.NotePrevented,
		&quiz.MailNotifications,
		&quiz.UniqueAnswers,
		&quiz.Super,
		&quiz.GroupID,
		&quiz.Name,
		&quiz.Description,
		&quiz.Config,
		&quiz.Status,
		&quiz.LimitAnswers,
		&quiz.DueTo,
		&quiz.TimeOfPassing,
		&quiz.Pausable,
		&quiz.Version,
		&quiz.VersionComment,
		pq.Array(&quiz.ParentIds),
		&quiz.CreatedAt,
		&quiz.UpdatedAt,
		&quiz.QuestionsCount,
		&quiz.AnswersCount,
		&quiz.AverageTimePassing,
		&quiz.SessionsCount,
	)
	if err != nil {
		return nil, err
	}

	return &quiz, nil
}
