package transfer

import (
	"context"
	"database/sql"
	"gitea.pena/PenaSide/hlog"
	"github.com/google/uuid"
	"github.com/lib/pq"
	"time"
	"transfer_to_clickhouse/models"
)

type QuizCore struct {
	db     *sql.DB
	logger hlog.Logger
}

func NewQuizCore(db *sql.DB, logger hlog.Logger) *QuizCore {
	return &QuizCore{
		db:     db,
		logger: logger,
	}
}

type AccountSql struct {
	ID        uuid.UUID `db:"id" json:"id"`
	UserID    string    `db:"user_id" json:"user_id"`
	Email     *string   `db:"email" json:"email"`
	CreatedAt time.Time `db:"created_at" json:"created_at"`
	Deleted   bool      `db:"deleted" json:"deleted"`
}

func (r *QuizCore) AccountEvents(ctx context.Context) error {
	query := "SELECT id, user_id, email, created_at, deleted FROM account"
	rows, err := r.db.QueryContext(ctx, query)
	if err != nil {
		return err
	}
	defer rows.Close()

	var accounts []AccountSql

	for rows.Next() {
		var account AccountSql
		if err := rows.Scan(&account.ID, &account.UserID, &account.Email, &account.CreatedAt, &account.Deleted); err != nil {
			return err
		}
		accounts = append(accounts, account)
	}

	if err := rows.Err(); err != nil {
		return err
	}

	for _, i := range accounts {
		r.logger.Emit(models.InfoAccountCreated{
			CtxUserID:    i.UserID,
			CtxAccountID: i.ID.String(),
		})
	}

	return nil
}

type Quiz struct {
	ID                 int64     `db:"id" json:"id"`
	Qid                uuid.UUID `db:"qid" json:"qid"`
	Accountid          string    `db:"accountid" json:"accountid"`
	Deleted            bool      `db:"deleted" json:"deleted"`
	Archived           bool      `db:"archived" json:"archived"`
	Fingerprinting     bool      `db:"fingerprinting" json:"fingerprinting"`
	Repeatable         bool      `db:"repeatable" json:"repeatable"`
	NotePrevented      bool      `db:"note_prevented" json:"note_prevented"`
	MailNotifications  bool      `db:"mail_notifications" json:"mail_notifications"`
	UniqueAnswers      bool      `db:"unique_answers" json:"unique_answers"`
	Super              bool      `db:"super" json:"super"`
	GroupID            int64     `db:"group_id" json:"group_id"`
	Name               string    `db:"name" json:"name"`
	Description        string    `db:"description" json:"description"`
	Config             string    `db:"config" json:"config"`
	Status             string    `db:"status" json:"status"`
	LimitAnswers       int32     `db:"limit_answers" json:"limit_answers"`
	DueTo              int32     `db:"due_to" json:"due_to"`
	TimeOfPassing      int32     `db:"time_of_passing" json:"time_of_passing"`
	Pausable           bool      `db:"pausable" json:"pausable"`
	Version            int16     `db:"version" json:"version"`
	VersionComment     string    `db:"version_comment" json:"version_comment"`
	ParentIds          []int32   `db:"parent_ids" json:"parent_ids"`
	CreatedAt          time.Time `db:"created_at" json:"created_at"`
	UpdatedAt          time.Time `db:"updated_at" json:"updated_at"`
	QuestionsCount     int32     `db:"questions_count" json:"questions_count"`
	AnswersCount       int32     `db:"answers_count" json:"answers_count"`
	AverageTimePassing int32     `db:"average_time_passing" json:"average_time_passing"`
	SessionsCount      int32     `db:"sessions_count" json:"sessions_count"`
}

func (r *QuizCore) QuizEvents(ctx context.Context) error {
	query := `
		SELECT id, qid, accountid, deleted, archived, fingerprinting, repeatable, note_prevented, 
			mail_notifications, unique_answers, super, group_id, name, description, config, status, 
			limit_answers, due_to, time_of_passing, pausable, version, version_comment, parent_ids, 
			created_at, updated_at, questions_count, answers_count, average_time_passing, sessions_count
		FROM quiz
	`

	rows, err := r.db.QueryContext(ctx, query)
	if err != nil {
		return err
	}
	defer rows.Close()

	var quizzes []Quiz

	for rows.Next() {
		var quiz Quiz
		if err := rows.Scan(
			&quiz.ID, &quiz.Qid, &quiz.Accountid, &quiz.Deleted, &quiz.Archived, &quiz.Fingerprinting,
			&quiz.Repeatable, &quiz.NotePrevented, &quiz.MailNotifications, &quiz.UniqueAnswers,
			&quiz.Super, &quiz.GroupID, &quiz.Name, &quiz.Description, &quiz.Config, &quiz.Status,
			&quiz.LimitAnswers, &quiz.DueTo, &quiz.TimeOfPassing, &quiz.Pausable, &quiz.Version,
			&quiz.VersionComment, pq.Array(&quiz.ParentIds), &quiz.CreatedAt, &quiz.UpdatedAt,
			&quiz.QuestionsCount, &quiz.AnswersCount, &quiz.AverageTimePassing, &quiz.SessionsCount); err != nil {
			return err
		}

		quizzes = append(quizzes, quiz)
	}

	if err := rows.Err(); err != nil {
		return err
	}

	for _, quiz := range quizzes {
		r.logger.Emit(models.InfoQuizCreated{
			CtxUserID: quiz.Accountid,
			CtxIDInt:  quiz.ID,
		})
	}

	for _, quiz := range quizzes {
		if quiz.Deleted == true {
			r.logger.Emit(models.InfoQuizDelete{
				CtxUserID: quiz.Accountid,
				CtxIDInt:  quiz.ID,
			})

		}

		if quiz.Status == "stop" {
			r.logger.Emit(models.InfoQuizStop{
				CtxUserID: quiz.Accountid,
				CtxIDInt:  quiz.ID,
			})
		}

		if quiz.Status == "start" {
			r.logger.Emit(models.InfoQuizPublish{
				CtxUserID: quiz.Accountid,
				CtxIDInt:  quiz.ID,
			})
		}
	}

	return nil
}

type Question struct {
	ID           int64     `db:"id" json:"id"`
	QuizID       int64     `db:"quiz_id" json:"quiz_id"`
	Title        string    `db:"title" json:"title"`
	Description  string    `db:"description" json:"description"`
	Questiontype string    `db:"questiontype" json:"questiontype"`
	Required     bool      `db:"required" json:"required"`
	Deleted      bool      `db:"deleted" json:"deleted"`
	Page         int16     `db:"page" json:"page"`
	Content      string    `db:"content" json:"content"`
	Version      int16     `db:"version" json:"version"`
	ParentIds    []int32   `db:"parent_ids" json:"parent_ids"`
	CreatedAt    time.Time `db:"created_at" json:"created_at"`
	UpdatedAt    time.Time `db:"updated_at" json:"updated_at"`
}

func (r *QuizCore) QuestionEvents(ctx context.Context) error {
	query := `SELECT id, quiz_id, title, description, questiontype, required, deleted, page, content, version, parent_ids, created_at, updated_at 
              FROM question`

	rows, err := r.db.QueryContext(ctx, query)
	if err != nil {
		return err
	}
	defer rows.Close()

	var questions []Question

	for rows.Next() {
		var question Question

		if err := rows.Scan(&question.ID, &question.QuizID, &question.Title, &question.Description, &question.Questiontype,
			&question.Required, &question.Deleted, &question.Page, &question.Content, &question.Version,
			pq.Array(&question.ParentIds), &question.CreatedAt, &question.UpdatedAt); err != nil {
			return err
		}

		questions = append(questions, question)
	}

	if err := rows.Err(); err != nil {
		return err
	}

	for _, question := range questions {
		accID, _ := r.getAccountIDByQuizID(ctx, question.QuizID)
		r.logger.Emit(models.InfoQuestionCreate{
			CtxUserID:     accID,
			CtxIDInt:      question.QuizID,
			CtxQuestionID: question.ID,
		})

		if question.Deleted == true {
			r.logger.Emit(models.InfoQuestionDelete{
				CtxUserID:     accID,
				CtxIDInt:      question.QuizID,
				CtxQuestionID: question.ID,
			})
		}
	}

	return nil
}

func (r *QuizCore) getAccountIDByQuizID(ctx context.Context, quizID int64) (string, error) {
	query := "select accountid from quiz where id = $1"

	var accountID string
	err := r.db.QueryRowContext(ctx, query, quizID).Scan(&accountID)
	if err != nil {
		return "", nil
	}

	return accountID, nil
}
